<?php
/*
 * Google Meet for LatePoint
 * Copyright (c) 2022-2023 TechXela (https://codecanyon.net/user/tech-xela). All Rights Reserved.
 *
 * LICENSE
 * -------
 * This software is furnished under license(s) and may be used and copied only in accordance with the terms of such
 * license(s) along with the inclusion of the above copyright notice. If you purchased this software through CodeCanyon,
 * please read the full license(s) at: https://codecanyon.net/licenses/standard
 */

if ( ! class_exists( 'TechXelaLatePointGoogleMeetMeetingHelper' ) ) :

	final class TechXelaLatePointGoogleMeetMeetingHelper implements TechXelaLatePointMeetingHelperIface {
		const cDV = [ 'conferenceDataVersion' => 1 ];

		private static function getMeetingName( $booking ) {
			return OsReplacerHelper::replace_all_vars( TechXelaLatePointGoogleMeetSettingsHelper::getMeetingNameTemplate(), [
				'customer' => $booking->customer,
				'agent'    => $booking->agent,
				'booking'  => $booking
			] );
		}

		private static function getFormattedEntryPointLabel( $booking, $entryPoint ) {
			return OsReplacerHelper::replace_all_vars(
				TechXelaLatePointGoogleMeetServiceHelper::getEntryPointLabel( $booking->service_id, $entryPoint ),
				[
					'customer' => $booking->customer,
					'agent'    => $booking->agent,
					'booking'  => $booking
				] );
		}

		public static function generateMeetingPin(): string {
			return OsUtilHelper::random_text( 'nozero', 4 );
		}

		public static function generateMeetingPassword(): string {
			return OsUtilHelper::random_text();
		}

		public static function createMeeting( OsBookingModel $booking, bool $updateIfExists = false ): void {
			$gClient    = OsGoogleCalendarHelper::get_authorized_client_for_agent( $booking->agent_id );
			$calendarId = OsGoogleCalendarHelper::get_selected_calendar_id_for_push( $booking->agent_id );
			$eventId    = $booking->get_meta_by_key( 'google_calendar_event_id', false );

			if ( $gClient && $calendarId && $eventId ) {
				$gService = new \LatePoint\GoogleCalendarAddon\Google_Service_Calendar( $gClient );

				try {
					$event = $gService->events->get( $calendarId, $eventId );

					if ( $updateIfExists && $event->getConferenceData() ) {
						self::updateEventFromUpdatedBooking( $event, $booking );
					} else {
						self::updateEventFromCreatedBooking( $event, $booking );
					}

					$event->setGuestsCanInviteOthers( TechXelaLatePointGoogleMeetServiceHelper::guestsCanInviteOthers( $booking->service_id ) );
					$event->setGuestsCanSeeOtherGuests( TechXelaLatePointGoogleMeetServiceHelper::guestsCanSeeOtherGuests( $booking->service_id ) );

					$event = $gService->events->update( $calendarId, $eventId, $event, self::cDV );

					if ( $event && $event->getConferenceData() ) {
						if ( ! $updateIfExists ) {
							self::updateEventFromUpdatedBooking( $event, $booking );
							$event = $gService->events->update( $calendarId, $eventId, $event, self::cDV );
						}

						if ( TechXelaLatePointGoogleMeetServiceHelper::groupBookingsEnabled( $booking->service_id ) ) {
							$bookings = TechXelaLatePointBookingHelper::getGroupBookings( $booking );
						}

						$bookings[] = $booking;
						$meeting    = $event->getConferenceData();

						foreach ( $bookings as $booking ) {
							$booking->save_meta_by_key( 'gmeet_meeting_id', $meeting->getConferenceId() );
							$booking->save_meta_by_key( 'gmeet_meeting_url', $meeting->getEntryPoints()[0]->getUri() );
							$booking->save_meta_by_key( 'gmeet_meeting_password', $meeting->getEntryPoints()[0]->getPassword() ?: "N/A" );
						}
					}
				} catch ( \Throwable $exception ) {
					TechXelaLatePointDebugHelper::logException( $exception, 'GOOGLE_MEET_CREATE_MEETING' );
				}
			}
		}

		public static function updateEventFromCreatedBooking( \LatePoint\GoogleCalendarAddon\Google_Service_Calendar_Event $event, OsBookingModel $booking ) {
			$conferenceData        = new \LatePoint\GoogleCalendarAddon\Google_Service_Calendar_ConferenceData();
			$conferenceSolutionKey = new \LatePoint\GoogleCalendarAddon\Google_Service_Calendar_ConferenceSolutionKey();
			$createRequest         = new \LatePoint\GoogleCalendarAddon\Google\Service\Calendar\CreateConferenceRequest();
			$meetingName           = self::getMeetingName( $booking );
			$meetingIconURL        = $booking->service->selection_image_url;

			$conferenceSolution = new \LatePoint\GoogleCalendarAddon\Google\Service\Calendar\ConferenceSolution();
			$conferenceSolution->setName( $meetingName );
			$conferenceSolution->setIconUri( $meetingIconURL );
			$conferenceData->setConferenceSolution( $conferenceSolution );

			$conferenceSolutionKey->setType( 'hangoutsMeet' );
			$createRequest->setConferenceSolutionKey( $conferenceSolutionKey );
			$createRequest->setRequestId( OsUtilHelper::random_text( 'alnum', 10 ) );
			$conferenceData->setCreateRequest( $createRequest );

			$event->setConferenceData( $conferenceData );
		}

		public static function updateEventFromUpdatedBooking( \LatePoint\GoogleCalendarAddon\Google_Service_Calendar_Event $event, OsBookingModel $booking ) {
			$conferenceData        = $event->getConferenceData();
			$meetingName           = self::getMeetingName( $booking );
			$conferenceEntryPoints = [];

			foreach ( $conferenceData->getEntryPoints() as $entryPoint ) {
				switch ( $entryPoint->getEntryPointType() ) {
					case 'video':
						$conferenceEntryPoints[] = $entryPoint;
						break;
					case 'phone':
						$phoneEntryPoint = $entryPoint;
						break;
					case 'sip':
						$sipEntryPoint = $entryPoint;
						break;
					case 'more':
						$moreEntryPoint = $entryPoint;
						break;
				}
			}

			if ( TechXelaLatePointGoogleMeetServiceHelper::isPhoneEntryPointEnabled( $booking->service_id ) ) {
				if ( ! isset( $phoneEntryPoint ) ) {
					$phoneEntryPoint = new \LatePoint\GoogleCalendarAddon\Google_Service_Calendar_EntryPoint();
					$phoneEntryPoint->setEntryPointType( 'phone' );
				}

				$phoneEntryPoint->setLabel( self::getFormattedEntryPointLabel( $booking, 'phone' ) ?:
					sprintf( esc_html_tx__( 'Dial in to %s meeting via Phone', 'google-meet-latepoint' ), $meetingName ) );
				$phoneEntryPoint->setUri( 'tel:' . OsUtilHelper::e164format( TechXelaLatePointGoogleMeetServiceHelper::getPhoneEntryPointUri( $booking->service_id ) ) );
				$phoneEntryPoint->setPin( TechXelaLatePointGoogleMeetServiceHelper::getPhoneEntryPointPin( $booking->service_id ) ?: self::generateMeetingPin() );

				$conferenceEntryPoints[] = $phoneEntryPoint;
			}

			if ( TechXelaLatePointGoogleMeetServiceHelper::isSipEntryPointEnabled( $booking->service_id ) ) {
				if ( ! isset( $sipEntryPoint ) ) {
					$sipEntryPoint = new \LatePoint\GoogleCalendarAddon\Google_Service_Calendar_EntryPoint();
					$sipEntryPoint->setEntryPointType( 'sip' );
				}

				$sipEntryPoint->setUri( 'sip:' . TechXelaLatePointGoogleMeetServiceHelper::getSipEntryPointUri( $booking->service_id ) );
				$sipEntryPoint->setPin( TechXelaLatePointGoogleMeetServiceHelper::getSipEntryPointPin( $booking->service_id ) ?: self::generateMeetingPin() );

				$conferenceEntryPoints[] = $sipEntryPoint;
			}

			if ( TechXelaLatePointGoogleMeetServiceHelper::isMoreEntryPointEnabled( $booking->service_id ) ) {
				if ( ! isset( $moreEntryPoint ) ) {
					$moreEntryPoint = new \LatePoint\GoogleCalendarAddon\Google_Service_Calendar_EntryPoint();
					$moreEntryPoint->setEntryPointType( 'more' );
				}

				$moreEntryPoint->setUri( TechXelaLatePointGoogleMeetServiceHelper::getMoreEntryPointUri( $booking->service_id ) );

				$conferenceEntryPoints[] = $moreEntryPoint;
			}

			$conferenceData->setEntryPoints( $conferenceEntryPoints );

			$event->setConferenceData( $conferenceData );
		}

		public static function getMeetingIdForBookingId( $bookingId ) {
			return OsMetaHelper::get_booking_meta_by_key( 'gmeet_meeting_id', $bookingId );
		}

		public static function getMeetingUrlForBookingId( $bookingId ) {
			return OsMetaHelper::get_booking_meta_by_key( 'gmeet_meeting_url', $bookingId );
		}

		public static function getMeetingPasswordForBookingId( $bookingId ) {
			return OsMetaHelper::get_booking_meta_by_key( 'gmeet_meeting_password', $bookingId );
		}

		public static function deleteMeetingForBookingId( $bookingId ): void {
			$meetingId = self::getMeetingIdForBookingId( $bookingId );
			if ( $meetingId ) {
				$booking    = new OsBookingModel( $bookingId );
				$gClient    = \OsGoogleCalendarHelper::get_authorized_client_for_agent( $booking->agent_id );
				$calendarId = \OsGoogleCalendarHelper::get_selected_calendar_id_for_push( $booking->agent_id );
				$eventId    = $booking->get_meta_by_key( 'google_calendar_event_id', false );

				if ( $gClient && $calendarId && $eventId ) {
					$gService = new \LatePoint\GoogleCalendarAddon\Google_Service_Calendar( $gClient );

					try {
						$event = $gService->events->get( $calendarId, $eventId );

						$event->setConferenceData( new \LatePoint\GoogleCalendarAddon\Google\Service\Calendar\ConferenceData() );

						$event = $gService->events->update( $calendarId, $eventId, $event, self::cDV );

						if ( $event && ! $event->getConferenceData() ) {
							$booking->delete_meta_by_key( 'gmeet_meeting_id' );
							$booking->delete_meta_by_key( 'gmeet_meeting_url' );
							$booking->delete_meta_by_key( 'gmeet_meeting_password' );
						}
					} catch ( \Throwable $exception ) {
						TechXelaLatePointDebugHelper::logException( $exception, 'GOOGLE_MEET_DELETE_MEETING' );
					}
				}
			}
		}
	}

endif;